"""
Objective functions
"""

from math import exp
import numpy as np

# Flags to pass to the objective functions
GET_FUNC = 'func'
GET_GRAD = 'grad'
GET_HESS = 'hess'

# Keep count of the number of calls
__COUNTS = {}

def ResetCounts():
  # ugly because we hae to support old python and there
  # is no defaultdict
  global __COUNTS
  __COUNTS = { }
  flags = [GET_FUNC, GET_GRAD, GET_HESS]
  for flag in flags:
    __COUNTS.setdefault(flag, 0)


def obja(x, flag):
  __COUNTS[flag] += 1
  if flag == GET_FUNC:
    return  x[0]**2 + 5*x[1]**2 + x[0] - 5*x[1]

  if flag == GET_GRAD:
    return np.array([2*x[0]+1, 10*x[1]-5])

  if flag == GET_HESS:
    return np.array([[2, 0],[0, 10]])

def objb(x, flag):
  __COUNTS[flag] += 1
  if flag == GET_FUNC:
    return x[0]**2+5*x[0]*x[1]+100*x[1]**2-x[0]+4*x[1]

  if flag == GET_GRAD:
    return np.array([2*x[0]+5*x[1]-1, 5*x[0]+200*x[1]+4])

  if flag == GET_HESS:
    return np.array([[2,5],[5,200]])

def objc(x, flag):
  __COUNTS[flag] += 1
  u = exp(x[0]+3*x[1]-0.1)
  v = exp(x[0]-3*x[1]-0.1)
  w = exp(-x[0]-0.1)
  if flag == GET_FUNC:
    return u+v+w

  if flag == GET_GRAD:
    return np.array([u+v-w, 3*u-3*v])

  if flag == GET_HESS:
    return np.array([[u+v+w, 3*u-3*v],[3*u-3*v,9*u+9*v]])

def objd(x, flag):
  __COUNTS[flag] += 1
  z = x[1]-x[0]**2
  if flag == GET_FUNC:
    return 100*z**2 + (1-x[0])**2

  if flag == GET_GRAD:
    return np.array([-400*x[0]*z + 2*x[0] - 2, 200*z])

  if flag == GET_HESS:
    return np.array([[1200*x[0]**2 - 400*x[1]+2, -400*x[0]],[-400*x[0], 200]])
